@@ -26,7 +26,7 @@ static int pixmapErrorTrap(disp, pErrorEvent)
     GotError = 1;
     XGetErrorText(disp, pErrorEvent->error_code, buf, MAXERRORLEN);
     fprintf(stderr, "serial #%d (request code %d) Got Error %s\n",
-	pErrorEvent->serial,
+	(int) pErrorEvent->serial,
 	pErrorEvent->request_code,
 	buf);
     return(0);
@@ -89,6 +89,7 @@ static unsigned int bitsPerPixelAtDepth(disp, scrn, depth)
 
   fprintf(stderr, "bitsPerPixelAtDepth: Can't find pixmap depth info!\n");
   cleanup(-1);
+  return (-1); /* To avoid -Wall warnings */
 }
      
 XImageInfo *imageToXImage(disp, scrn, visual, ddepth, image, private_cmap, fit,
@@ -103,7 +104,7 @@ XImageInfo *imageToXImage(disp, scrn, visual, ddepth, image, private_cmap, fit,
      unsigned int  back;
      unsigned int  verbose;
 { Pixel        *index, *redvalue, *greenvalue, *bluevalue;
-  unsigned int  a, b, newmap, x, y, linelen, dpixlen, dbits, again_flag = 0;
+  unsigned int  a, b=0, newmap, x, y, linelen, dpixlen, dbits, again_flag=0;
   XColor        xcolor;
   XGCValues     gcv;
   XImageInfo   *ximageinfo;
@@ -168,7 +169,7 @@ XImageInfo *imageToXImage(disp, scrn, visual, ddepth, image, private_cmap, fit,
       unsigned int redcolors, greencolors, bluecolors;
       unsigned int redstep, greenstep, bluestep;
       unsigned int redbottom, greenbottom, bluebottom;
-      unsigned int redtop, greentop, bluetop;
+      unsigned int redtop=0, greentop=0, bluetop=0;
 
       redvalue= (Pixel *)lmalloc(sizeof(Pixel) * 256);
       greenvalue= (Pixel *)lmalloc(sizeof(Pixel) * 256);
@@ -412,7 +413,7 @@ again:
 	XUngrabServer(disp);
       }
       else {
-	for (b= 0; b < a; b++) {
+	for (b = 0; b < a; b++) {
 	  xcolor.pixel= *(index + b);
 	  xcolor.red= *(image->rgb.red + b);
 	  xcolor.green= *(image->rgb.green + b);
