@@ -26,10 +26,10 @@
  * SUCH DAMAGE.
  */
 /*
- * $Id: mgp.c,v 1.129 2003/06/24 05:26:33 kazu Exp $
+ * $Id: mgp.c,v 1.138 2004/09/24 16:42:51 nishida Exp $
  */
 
-static char *mgp_version = "1.10a (20030624)";
+static char *mgp_version = "1.11b (20040925)";
 
 #include "mgp.h"
 #ifdef USE_SETLOCALE
@@ -196,6 +196,9 @@ main(argc, argv)
 	int argc;
 	char **argv;
 {
+	int i, j;              /* counters */
+	int tmp_argc;          /* number of current arguments */
+	char **tmp_argv;       /* manipulated 'argv' */
 	int opt;
 	extern char *optarg;
 	extern int optind;
@@ -221,6 +224,37 @@ main(argc, argv)
 		mgpwdir = p;
 	}
 
+	/*
+	 * check for the argument '--title' to set the window's title:
+	 * go through the whole 'argv' and cut off this option since it
+	 * will not be accepted by getopt()
+	 * default title is 'MagicPoint'
+	 */
+	tmp_argv=(char**)malloc(argc*sizeof(char*));
+	tmp_argc=argc;
+	i=j=0;
+	while (i < argc)
+	{
+		if (strcmp(argv[i], "--title") == 0)
+		{
+			tmp_argc--;
+			if (++i < argc)
+			{
+				mgp_wname=strdup(argv[i]);
+				tmp_argc--;
+			}
+		}
+		else
+			tmp_argv[j++]=strdup(argv[i]);
+		i++;
+	}
+	/* set title to default if not set by user */
+	if (argc-1 <= tmp_argc)
+		mgp_wname=strdup("MagicPoint");
+
+	argv=tmp_argv;
+	argc=tmp_argc;
+
 #define ACCEPTOPTS	"Bd:vVob:c:eg:f:hlGp:qt:Q:PSUT:D:CORw:X:x:nF:E:"
 	while ((opt = getopt(argc, argv, ACCEPTOPTS)) != -1) {
 #undef ACCEPTOPTS
@@ -344,6 +378,8 @@ main(argc, argv)
 				mgp_flag |= FL_NOFREETYPE;
 			else if (strcmp(optarg, "xft") == 0)
 				mgp_flag |= FL_NOXFT;
+			else if (strcmp(optarg, "m17n") == 0)
+				mgp_flag |= FL_NOM17N;
 			else {
 				fprintf(stderr, "unknown rendering engine %s\n",
 					optarg);
@@ -404,7 +440,7 @@ main(argc, argv)
     }
 
 	init_win1(xgeometry);
-	strncpy(buf, mgp_fname, sizeof(buf));
+	strlcpy(buf, mgp_fname, sizeof(buf));
 	if ((p = rindex(buf, '/'))) {
 		*p = '\0';
         	Paths[NumPaths++]= expandPath(buf);
@@ -509,13 +545,19 @@ genhtml(start_page)
 		sprintf(buf, "xwintoppm -silent -name MagicPoint | "
 			"%s %s > %s/mgp%05d.%s",
 			convdb[inum][1], childdebug, dumpdir, page, EXT);
-		system(buf);	/*XXX security hole*/
+		if (system(buf) < 0){	/*XXX security hole*/
+			fprintf(stderr, "system() failed for %s", buf);
+			exit(-1);
+		}
 		fprintf(stderr, "(thumbnail)");
 		sprintf(buf, "%s %s/mgp%05d.%s | "
 			"pnmscale 0.25 | %s %s > %s/mgp%05d.idx.%s",
 			convdb[inum][2], dumpdir, page, EXT, convdb[inum][1], childdebug,
 			dumpdir, page, EXT);
-		system(buf);	/*XXX security hole*/
+		if (system(buf) < 0){	/*XXX security hole*/
+			fprintf(stderr, "system() failed for %s", buf);
+			exit(-1);
+		}
 
 		/*
 		 * dump out html file
@@ -559,7 +601,7 @@ genhtml(start_page)
 				page);
 		}
 		fprintf(html, "<HR>Generated by "
-		    "<A HREF=\"http://www.mew.org/mgp/\">MagicPoint</A>\n"
+		    "<A HREF=\"http://member.wide.ad.jp/wg/mgp/\">MagicPoint</A>\n"
 		    "</BODY></HTML>\n");
 		fclose(html);
 
@@ -628,7 +670,7 @@ txtdone:;
 	}
 	fprintf(html, "<HR>\n");
 	fprintf(html, "Generated by "
-		"<A HREF=\"http://www.mew.org/mgp/\">"
+		"<A HREF=\"http://member.wide.ad.jp/wg/mgp/\">"
 		"MagicPoint</A>\n");
 	fprintf(html, "<BR>\n</BODY></HTML>\n");
 	fclose(html);
@@ -691,6 +733,7 @@ mgp_usage(name)
 	fprintf(stderr, "\t-P: print stderr from image conversion tools (by default it's discarded)\n");
 	fprintf(stderr, "\t-V: Be verbose\n");
 	fprintf(stderr, "\t-X <gsdevice>: ghostscript device to use\n");
+	fprintf(stderr, "\t--title <title>: Set window title\n");
 
 	exit(0);
 }
@@ -1059,6 +1102,10 @@ repaint:;
 						0, 0, window_width, window_height, 0, 0);
 				break;
 
+			case XK_w:
+				toggle_fullscreen();		
+				break;
+
 			default:
 				number = 0;
 				break;
