@@ -26,7 +26,7 @@
  * SUCH DAMAGE.
  */
 /*
- * $Id: mgp.h,v 1.134 2003/06/06 07:18:25 nishida Exp $
+ * $Id: mgp.h,v 1.140 2004/09/07 16:57:42 nishida Exp $
  */
 
 #include <stdio.h>
@@ -62,7 +62,8 @@
 #include <X11/cursorfont.h>
 #include <X11/Xatom.h>
 #ifdef USE_XFT2
-#include <freetype/freetype.h>
+#include <ft2build.h>
+#include FT_FREETYPE_H
 #include <fontconfig/fontconfig.h>
 #include <X11/Xft/Xft.h>
 #ifdef HAVE_ICONV
@@ -103,12 +104,21 @@
 #undef	TTY_KEYINPUT
 #endif
 
+#ifdef USE_M17N
+#include <m17n-gui.h>
+#include <m17n-misc.h>
+#include <m17n-X.h>
+#endif
+
 #define DEFAULT_FORE	"yellow"
 #define DEFAULT_BACK	"black"
 #define PAGELIST_FONT	"a14"
 #define PAGELIST_KFONT	"k14"
 
 #define DEFAULT_CHARSIZE	10	/* 10% of height */
+#define DEFAULT_SUPSCALE	0.6
+#define DEFAULT_SUPOFF		0.4
+#define DEFAULT_SUBOFF		0.15
 #define DEFAULT_HGAP		0
 #define DEFAULT_VGAP		15
 #define DEFAULT_BQUALITY	100
@@ -178,6 +188,7 @@
 #define	FL_GLYPHEDGE	0x4000
 #define	FL_FRDCACHE		0x8000
 #define	FL_NOXFT		0x10000
+#define	FL_NOM17N		0x20000
 
 /* 	page attribute flags 	*/
 #define PGFLAG_NODEF	0x01	/* nodefault */
@@ -198,6 +209,17 @@ struct ctrl_int {
 	u_int ct_value;
 };
 
+struct ctrl_int2 {
+       u_int ct_value1;
+        u_int ct_value2;
+};
+
+struct ctrl_int3 {
+       u_int ct_value1;
+        u_int ct_value2;
+        u_int ct_value3;
+	};
+
 struct ctrl_long {
 	u_long ct_value;
 };
@@ -223,6 +245,7 @@ struct ctrl_image {
 	u_int ct_ximagesize;
 	u_int ct_yimagesize;
 	u_int ct_zoomflag;
+	u_int ct_raise;
 #define Z_XMASK		0x0f
 #define Z_YMASK		0xf0
 #define Z_YSHIFT	4
@@ -289,6 +312,8 @@ struct ctrl {
 	union {
 		struct ctrl_double ctrl_double;
 		struct ctrl_int ctrl_int;
+                struct ctrl_int2 ctrl_int2;
+                struct ctrl_int3 ctrl_int3;        
 		struct ctrl_long ctrl_long;
 		struct ctrl_char ctrl_char;
 		struct ctrl_char2 ctrl_char2;
@@ -304,6 +329,11 @@ struct ctrl {
 
 #define ctf_value	ct_val.ctrl_double.ct_value
 #define cti_value	ct_val.ctrl_int.ct_value
+#define cti2_value1	ct_val.ctrl_int2.ct_value1
+#define cti2_value2	ct_val.ctrl_int2.ct_value2
+#define cti3_value1	ct_val.ctrl_int3.ct_value1
+#define cti3_value2	ct_val.ctrl_int3.ct_value2
+#define cti3_value3	ct_val.ctrl_int3.ct_value3
 #define ctl_value	ct_val.ctrl_long.ct_value
 #define ctc_value	ct_val.ctrl_char.ct_value
 #define ctc2_value1	ct_val.ctrl_char2.ct_value1
@@ -313,6 +343,7 @@ struct ctrl {
 #define ctm_ximagesize	ct_val.ctrl_image.ct_ximagesize
 #define ctm_yimagesize	ct_val.ctrl_image.ct_yimagesize
 #define ctm_zoomflag	ct_val.ctrl_image.ct_zoomflag
+#define ctm_raise	ct_val.ctrl_image.ct_raise
 #define ctd_colors	ct_val.ctrl_grad.colors
 #define ctd_g_colors	ct_val.ctrl_grad.ct_g_colors
 #define ctd_numcolor	ct_val.ctrl_grad.ct_numcolor
@@ -411,9 +442,13 @@ struct render_state {
 	u_int repaint;
 	int maxascent;
 	int maxdescent;
+	int maxflheight;
+	int max_lineascent;   /* max size above baseline ignoring supscript */
+	int max_linedescent;  /* max size below baseline ignoring subscript */
 	u_int linewidth;
 	u_int brankline;
 	u_int opaque;
+	u_int charoff;
 	struct ctrl *xfont;
 	struct render_object *obj, *objlast;
 };
@@ -429,6 +464,7 @@ struct render_object {
 #define VL_CENTER	1
 #define VL_TOP		2
 #define VL_BOTTOM	3
+#define VL_ICENTER	4
 	enum {
 #ifdef VFLIB
 		O_VFONT,
@@ -443,7 +479,10 @@ struct render_object {
 #endif /* MNG */
 #ifdef USE_XFT2
 		O_XTFONT,
-#endif /* MNG */
+#endif 
+#ifdef USE_M17N
+		O_M17NTEXT,
+#endif 
 		O_ICON
 	} type;
 	u_long fore;
@@ -487,6 +526,13 @@ struct render_object {
 			int size;
 			int charset16;
 		} xftfont;
+#endif
+#ifdef USE_M17N
+		struct {
+			MText *mt;
+			MFrame *drawframe;
+			int len;
+		} m17ntext;
 #endif
 	} data;
 };
@@ -566,6 +612,7 @@ extern int verbose;
 extern u_int maxpage;
 extern u_int cur_page;
 extern char *mgp_fname;
+extern char *mgp_wname;
 
 extern u_int parse_error;
 extern u_int parse_debug;
@@ -585,6 +632,10 @@ extern struct alloc_color back_clr;
 extern struct alloc_color font_clr;
 
 extern u_int char_size[2];
+extern u_int nonscaled_size[2];
+extern float sup_scale;
+extern float sup_off;
+extern float sub_off;
 extern u_int horiz_gap[2];
 extern u_int vert_gap[2];
 extern u_int depth;
@@ -672,6 +723,7 @@ extern void cleanup __P((int));
 #ifdef TTY_KEYINPUT
 extern void try_enable_ttykey __P((void));
 #endif
+extern void reset_background_pixmap();
 
 /*draw.c*/
 extern void state_goto __P((struct render_state *, u_int, int));
@@ -727,6 +779,7 @@ extern int get_color __P((char *, u_long *));
 extern struct g_color *name2gcolor __P((char *));
 extern void regist_alloc_colors __P((struct alloc_color *, u_long *, int));
 extern void free_alloc_colors __P((struct alloc_color *));
+extern void toggle_fullscreen();
 
 /* background.c */
 extern double cdist __P((int, int, int, int, int, int));
@@ -744,6 +797,9 @@ extern struct tfont *tfc_get __P((u_int, u_int, int, char *, int));
 extern void tfc_setfont __P((char *, int, char *));
 extern XImage *tfc_image __P((struct tfont *, long, long, XImage *, int, int));
 
+/* unimap.c */
+extern void latin_unicode_map_init();
+
 /* embed.c */
 extern char *embed_fname __P((char *));
 extern void embed_file __P((FILE *, struct ctrl *, int *));
@@ -772,3 +828,13 @@ extern void usleep __P((u_int));
 #define COLOR_BUGFIX
 #endif
 
+#ifdef USE_M17N
+void M17N_init __P((void));
+void M17N_set_size __P((int));
+void M17N_set_color __P((u_long));
+MText * M17N_gen_mtext __P((u_char *));
+void M17N_process_direc __P((char *, char *));
+void M17N_draw_string __P((struct render_state *, char *));
+char * M17N_draw_fragment __P((struct render_state *, u_char *, u_int));
+void M17N_draw_object __P((struct render_object *, Drawable, int, int));
+#endif
