@@ -26,7 +26,7 @@
  * SUCH DAMAGE.
  */
 /*
- * $Id: parse.c,v 1.93 2003/06/14 10:22:56 nishida Exp $
+ * $Id: parse.c,v 1.98 2004/07/08 06:09:54 nishida Exp $
  */
 
 #include "mgp.h"
@@ -42,6 +42,7 @@ static void thirdpass __P((void));
 static void debug __P((void));
 static int define_font __P((struct ctrl *));
 static struct ctrl *find_font __P((char *));
+static char* clear_esc __P((char*));
 
 /*image*/
 extern int findImage __P((char *, char *));
@@ -133,6 +134,109 @@ cleanup_file()
 	}
 }
 
+/*
+ * clear the given data from escape sequences, so identifying
+ * a character by its hexadecimal value is possible by using \xHH
+ */
+static char*
+clear_esc(s)
+	char* s;
+{
+	char* p;
+	int len;
+	int i;
+	int j;
+	int k;
+	int error = 0;
+	int unicode = 0;
+	int chars;
+	unsigned int code;
+
+	i = j = 0;
+	len = strlen(s);
+	p = (char*)malloc(len + 1);
+
+	while (i < len)
+	{
+		code = 0x0;
+
+		/*
+		 * expecting either beginning of unicode or
+		 * an ascii-character (may be an escape sequence)
+		 */
+		if (!unicode) {
+			/* beginning of unicode */
+			if (s[i] == 0x1b && s[i+1] == '$' &&
+				'@' <= s[i+2] && s[i+2] < 'C') {
+				unicode = 1;
+
+				for (chars = 0; chars < 3; chars++)
+					p[j++] = s[i++];
+
+				continue;
+			}
+
+			/* backslash */
+			if (s[i] == 0x5c) {
+				i++;
+	
+				/* any escape sequence */
+				if (s[i] != 0x5c) {
+
+					 /* hexadecimal token: \xHH */
+					if ((s[i] == 'x' || s[i] == 'X') && i+2 < len) {
+						i++;
+	
+						if (isxdigit(s[i]) && isxdigit(s[i+1])) {
+							for (k = 2; k > 0; k--, i++) {
+								code <<= 4;
+	
+								if (isdigit(s[i]))
+									code += s[i] - 48;
+								else
+									code += tolower(s[i]) - 87;
+							}
+						} else
+							error = 1;
+					} else
+						error = 1;
+				}
+			}
+			if (error)
+				return (char*) NULL;
+
+			if (code == 0x0)
+				p[j++] = s[i++];
+			else
+				p[j++] = code;
+		} else {
+			/*
+			 * expecting either the end of unicode or
+			 * a part of a unicode character
+			 */
+
+			/* end of unicode */
+			if (s[i] == 0x1b && s[i+1] == '(' &&
+				(s[i+2] == 'B' || s[i+2] == 'J')) {
+				unicode = 0;
+
+				for (chars = 0; chars < 3; chars++)
+					p[j++] = s[i++];
+				
+				continue;
+			}
+
+			/* just take the unicode character */
+			p[j++] = s[i++];
+		}
+	}
+	p[j] = 0x0;
+	strcpy(s, p);
+	free(p);
+
+	return s;
+}
+	
 static struct ctrl *
 parse_text(p, page)
 	char *p;
@@ -437,7 +541,8 @@ command:
 			continue;
 		}
 	    {
-		char *p, *q;
+		char *p, *q, *s;
+		int cnt;   /* counter for backslashes at the end of a line */
 
 		p = buf + strlen(buf);
 		if (buf < p && p[-1] == '\n') {
@@ -445,6 +550,22 @@ command:
 			*p = '\0'; 
 		}
 		while (buf < p && p - buf < sizeof(buf) && p[-1] == '\\') {
+
+			/*
+			 * check for quoted backslashes at the end of a line:
+			 * if it is so, then do NOT ignore the line break and
+			 * continue as usual
+			 */
+			s = p;
+			cnt = 0;
+			while (buf < s && s[-1] == '\\') {
+				s--;
+				cnt++;
+			}
+
+			if (cnt % 2 == 0)
+				break;
+
 			p--;
 			if (fgets(buf2, sizeof(buf) - (p - buf), fp) == NULL)
 				break;
@@ -453,6 +574,7 @@ command:
 			while (*q && isspace(*q))
 				q++;
 			strncpy(p, q, sizeof(buf) - (p - buf));
+			p[sizeof(buf) - (p - buf) -1] = '\0';
 			p += strlen(p);
 			if (buf < p && p[-1] == '\n') {
 				p--;
@@ -462,10 +584,7 @@ command:
 	    }
 		if (buf[0] == '#')
 			continue;
-#if 1 /* escape # by backslash */
-		if (buf[0] == 0x5c && (buf[1] == 0x5c || buf[1] == 0x23))
-			memcpy(&buf[0], &buf[1], strlen(buf));
-#endif
+
 		if (buf[0] == '%') {
 			/* this is directive */
 			int pb;
@@ -482,7 +601,7 @@ command:
 			lex_init(buf + 1);
 			if (yyparse() || n_errors) {
 				fprintf(stderr,
-					"%s:%d: fatal syntax error detected\n",
+					"%s:%d: fatal syntax error detected !!!\n",
 					filename, lineno);
 				exit(-1);
 			}
@@ -664,6 +783,27 @@ command:
 			}
 		} else {
 			/* this is data */
+
+			/*
+			 * escape # and % by backslash (at the beginning
+			 * of a line)
+			 */
+			if (buf[0] == 0x5c && (buf[1] == 0x23 ||
+						buf[1] == 0x25))
+				memcpy(&buf[0], &buf[1], strlen(buf));
+
+			/*
+			 * clear escape sequences
+			 */
+#if 1
+			if (clear_esc(buf) == NULL) {
+				fprintf(stderr, "%s:%d: unknown escape"
+						" sequence\n",
+						filename, lineno);
+				exit(-1);
+			}
+#endif
+
 			cp = parse_text(buf, *page);
 			if (cp) {
 				ch = &page_control[*page][*line];
@@ -834,6 +974,24 @@ thirdpass()
 	tmpstr[5] = ctlalloc1(CTL_XFONT2);
 	tmpstr[5]->ctc2_value1 = strdup("k14");
 	tmpstr[5]->ctc2_value2 = strdup("jisx0208.1983-*");
+#ifdef USE_M17N
+	tmpstr[6] = ctlalloc1(CTL_M17N);
+	tmpstr[6]->ctc2_value1 = strdup("fontset");
+	tmpstr[6]->ctc2_value2 = strdup("default");
+	tmpstr[7] = ctlalloc1(CTL_M17N);
+	tmpstr[7]->ctc2_value1 = strdup("family");
+	tmpstr[7]->ctc2_value2 = strdup("default");
+	tmpstr[8] = ctlalloc1(CTL_M17N);
+	tmpstr[8]->ctc2_value1 = strdup("language");
+	tmpstr[8]->ctc2_value2 = strdup("default");
+#else
+	tmpstr[6] = ctlalloc1(CTL_NOOP);	
+	tmpstr[6]->ctc_value = strdup("");
+	tmpstr[7] = ctlalloc1(CTL_NOOP);	
+	tmpstr[7]->ctc_value = strdup("");
+	tmpstr[8] = ctlalloc1(CTL_NOOP);	
+	tmpstr[8]->ctc_value = strdup("");
+#endif
 	tmplong[0] = ctlalloc1(CTL_FORE);
 	get_color(DEFAULT_FORE, &tmplong[0]->ctl_value);
 	tmplong[1] = ctlalloc1(CTL_BACK);
@@ -848,7 +1006,7 @@ thirdpass()
 	tmpint[2]->cti_value = DEFAULT_VGAP;
 	tmpint[3] = ctlalloc1(CTL_QUALITY);
 	tmpint[3]->cti_value = DEFAULT_BQUALITY;
-#ifdef XFT2
+#ifdef USE_XFT2
 	tmpint[4] = ctlalloc1(CTL_OPAQUE);
 	tmpint[4]->cti_value = DEFAULT_OPAQUE;
 #endif
@@ -1045,8 +1203,20 @@ thirdpass()
 			continue;
 		line = page_attribute[page].pg_linenum;
 		for (l = 0; l <= line; l++) {
+			int contseen;
+			contseen = 0;	
 			ch = &page_control[page][l];
-			if (default_control[l]) {
+			/* 
+			 * if this line contains CTL_CONT, we don't add 
+			 * default directive to this line 
+			 */
+			for (cp = page_control[page][l]; cp; cp = cp->ct_next) {
+				if (cp->ct_op == CTL_CONT) {
+					contseen++;
+					break;
+				}
+			}
+			if (default_control[l] && !contseen) {
 				ctlinsert(ch, ctlcopy(default_control[l]));
 			}
 		}
