@@ -26,19 +26,26 @@
  * SUCH DAMAGE.
  */
 /*
- * $Id: print.c,v 1.102 2003/05/15 19:35:41 nishida Exp $
+ * $Id: print.c,v 1.106 2004/07/28 13:36:28 nishida Exp $
  */
 /*
  * Paper size selection code is based on psutil.c by Angus J. C. Duggan
  */
 
 #include "mgp.h"
+#ifdef USE_SETLOCALE
+#ifdef HAVE_LOCALE_H
+#include <locale.h>
+#endif
+#endif
 
 static u_int align = AL_LEFT;
 static char *curprefix = NULL;
 static char *tabprefix = NULL;
 static u_int lineheight;
 static u_int linewidth;
+static int char_off = 0;	/* y-offset of the chars (relative to the
+				   current line, used for super/subscript */
 
 #define DEFAULT_PAPER_SIZE	"a4"
 #define PRINT_ASCIIFONT		"Times-Roman"
@@ -52,11 +59,15 @@ static char outputfile[MAXVALLEN];
 static int colorps = 0;
 static int painticon = 0;
 static u_int curlinenum;
-static u_int curpagenum;
+static u_int curpagenum=0;
 static u_int xprefix = 0;
 static u_int tabxprefix = 0;
 static u_int valign = VL_BOTTOM;
 
+/* by Arnd Schmitter 23.07.2004 */
+static int lg; /* Counts the Recursionlevel in print_page() for new Pause-Mode */
+static int PauseMode=0; /* Should Pause-Statements splitet into multiple Pages ? */
+
 #ifdef FREETYPE
 static u_int usetfont = 0;
 #endif
@@ -73,6 +84,7 @@ static int nimagepool = 0;
 static struct textpool {
 	int pfx;
 	int xoffset;
+	int yoffset;     /* offset for super/subscript */
 	int xsiz;
 	struct fontmap *font;
 	int size;
@@ -114,6 +126,10 @@ static struct fontmap {
 	{ CTL_VFONT, KANJI, "*",		"Ryumin-Light-H" },
 #endif /*VFLIB*/
 #ifdef FREETYPE
+	{ CTL_TFONT, ASCII, "schlbk.ttf",	"NewCenturySchlbk-Roman" },
+	{ CTL_TFONT, ASCII, "schlbki.ttf",	"NewCenturySchlbk-Italic" },
+	{ CTL_TFONT, ASCII, "schlbkbd.ttf",	"NewCenturySchlbk-Bold" },
+	{ CTL_TFONT, ASCII, "schlbkbi.ttf",	"NewCenturySchlbk-BoldItalic" },
 	{ CTL_TFONT, ASCII, "times.ttf",	"Times-Roman" },
 	{ CTL_TFONT, ASCII, "timesi.ttf",	"Times-Italic" },
 	{ CTL_TFONT, ASCII, "timesbd.ttf",	"Times-Bold" },
@@ -126,6 +142,7 @@ static struct fontmap {
 	{ CTL_TFONT, ASCII, "couri.ttf",	"Courier-Oblique" },
 	{ CTL_TFONT, ASCII, "courbd.ttf",	"Courier-Bold" },
 	{ CTL_TFONT, ASCII, "courbi.ttf",	"Courier-BoldOblique" },
+	{ CTL_TFONT, ASCII, "symbol.ttf",	"Symbol" },
 	{ CTL_TFONT, ASCII, "*",		"Helvetica" },	/*last resort*/
 #ifdef FREETYPE_CHARSET16
 	{ CTL_TMFONT, KANJI, "wadalab-gothic.ttf", "GothicBBB-Medium-H" },
@@ -266,7 +283,7 @@ static void scan_font __P((u_int));
 static void print_page __P((u_int));
 static void print_init __P((void));
 static void print_pageinit __P((u_int));
-static void process_direc_print __P((struct ctrl *));
+static void process_direc_print __P((struct ctrl *cp));
 static void scan_backcolor __P((u_int));
 static void line_start __P((void));
 static void line_end __P((void));
@@ -292,6 +309,7 @@ static struct fontmap *findfont __P((int, int, char *));
 static struct fontmap *findttfont __P((int, int, char *));
 static char *fontstring __P((struct fontmap *));
 static void loadfont __P((struct fontmap *, char *));
+static int  count_pause __P((u_int page));   /* Added 23.07.2004 by Arnd Schmitter */
 
 /* image/ *.[ch] */
 extern char *Paths[];
@@ -324,6 +342,9 @@ print_out()
 	window_height = w_height;
 
 	char_size[0] = window_height * DEFAULT_CHARSIZE / 100;
+	sup_off = DEFAULT_SUPOFF;
+	sub_off = DEFAULT_SUBOFF;
+	sup_scale = DEFAULT_SUPSCALE;
 
 	if (outputfile[0]) {
 		if ((fp = fopen(outputfile, "w")) == NULL) {
@@ -338,7 +359,10 @@ print_out()
 		scan_font(i);
 	}
 	for (i = 1; i <= maxpage; i ++)
-		print_page(i);
+	  {
+	    lg=0; /* Set the recursionlevel to 0 for each new page (Arnd Schmitter 23.07.2004) */
+	    print_page(i);
+	  }
 
 	fclose(fp);
 
@@ -411,7 +435,6 @@ scan_font(page)
 			case CTL_XFONT:
 				fprintf(stderr, "obsolete directive XFONT\n");
 				exit(1);
-
 			case CTL_TEXT:
 				if (strstr(cp->ctc_value, "\033$B")
 				 || strstr(cp->ctc_value, "\033$@")) {
@@ -442,6 +465,32 @@ scan_font(page)
 	memset(curfont, 0, sizeof(curfont));
 }
 
+/*
+ * This Function counts the Number of Pause-Statements.
+ */
+static int 
+count_pause(page)
+     u_int page;
+{
+  int count=0;
+  u_int line;
+  struct ctrl*cp;
+  for (line = 0;
+       line <= page_attribute[page].pg_linenum;
+       line++) {
+    curlinenum = line;
+    /* process normal control */
+    for (cp = page_control[page][line]; cp; cp = cp->ct_next)
+      {
+	if(cp->ct_op == CTL_PAUSE)
+	  {
+	    count++;
+	  }
+      }
+  }
+  return count;
+}
+
 static void
 print_page(page)
 	u_int page;
@@ -449,9 +498,21 @@ print_page(page)
 	u_int line;
 	struct ctrl *cp;
 
-	curpagenum = page;
-	scan_backcolor(curpagenum);
+	int ret=0;  /* A Pause Command was found */
+	int lc = 0; /* Counter to prevent infinite recursion loops at Pause-Statements */
+	int pcount; /* Stores the Number of Pause Statements for a Page */
+
+
+	/* Modified by Arnd Schmitter 23.07.2004:
+     * when in PauseMode for emulationg %pause in PS file (option "-m")
+     * this function is called recursively to print the beginning of
+     * the page again up to current %pause statement
+     */
+	/*curpagenum = page;*/
+	curpagenum++;
+	scan_backcolor(page);
 	print_pageinit(curpagenum);
+	/* End of Modification */
 
 	for (line = 0;
 	     line <= page_attribute[page].pg_linenum;
@@ -459,11 +520,32 @@ print_page(page)
 		curlinenum = line;
 		/* process normal control */
 		for (cp = page_control[page][line]; cp; cp = cp->ct_next)
-			process_direc_print(cp);
+		  {
+		    if ((PauseMode) && (cp->ct_op == CTL_PAUSE))
+		      if (lc >= lg)
+			ret=1;
+		      else 
+			(lc)++;
+		    else 
+		      process_direc_print(cp);
+		    
+		    if (ret) goto pause; /* If a Pause has been found exit Loop immedeatly */
+		  }
 	}
-
+ pause:
 	fprintf(fp, "grestore\n\n");
 	fprintf(fp, "showpage\n\n");
+	/* If a Pause has been found -> Restart this Page */
+	if (ret)
+	  {
+	    lg++;
+	    /* If we are in Pause-Mode check if we are called because of the last CTL_PAUSE at End of Page */
+	    /*   At the Endo of the Last Page is no PAUSE Statemtent. So we have to use all Statements in the
+	     Page */
+	    pcount=count_pause(page);
+	    if ((pcount != lg)||(page == (maxpage)))
+	      print_page(page);
+	  }
 }
 
 static void
@@ -472,10 +554,25 @@ print_init()
 	char **p;
 	char *aligns[] = { "center", "left", "right", NULL };
 
+
+	/* Added by Arnd Schmitter 23.07.2004 */
+	/*   If we split Pages with Pause Statements the number of absolute Pages gets greater then maxpage */
+	u_int i;
+	int absolute_pages = maxpage+1;
+	if (PauseMode)
+	  {
+	    for (i = 0; i < maxpage; i++)
+	      {
+		absolute_pages+=count_pause(i)-1;
+	      }	   
+	  }
+
+	/*fprintf(stderr,"Pages %d\n",absolute_pages);*/
+
 	fprintf(fp, "%%!PS-Adobe-2.0\n");
 	fprintf(fp, "%%%%Creator: mgp2ps\n");
 	fprintf(fp, "%%%%Title: %s\n", mgp_fname);
-	fprintf(fp, "%%%%Pages: %d\n", maxpage);
+	fprintf(fp, "%%%%Pages: %d\n", absolute_pages);
 	fprintf(fp, "%%%%BoundingBox: 0 0 %d %d\n", paper->height, paper->width);
 	fprintf(fp, "%%%%DocumentPaperSizes: %s\n", paper->name);
 	fprintf(fp, "%%%%Orientation: Landscape\n");
@@ -493,7 +590,7 @@ print_init()
 	fprintf(fp, "/setymin {dup ymin lt {dup /ymin exch def} if pop} def\n");
 	fprintf(fp, "/calcy {dup ( ) eq\n"
 		"  {0 setymax 0 setymin pop}\n"
-		"  {false charpath flattenpath pathbbox setymax pop setymin pop}\n"
+		"  {pop charsize 0.7 mul setymax charsize -0.15 mul setymin}\n"
 		"  ifelse\n"
 		"} def\n");
 
@@ -657,6 +754,7 @@ process_direc_print(cp)
 		fprintf(fp, "%d setcharsize\n", char_size[0]);
 		break;
 
+	  
 #ifdef VFLIB
 	case CTL_VFONT:
 		font = findfont(cp->ct_op, KANJI, cp->ctc_value);
@@ -786,6 +884,13 @@ process_direc_print(cp)
 		print_line(cp->ctc_value, 0);
 		if (align == AL_LEFTFILL0)
 			align = AL_LEFTFILL1;
+
+		/* disable super/subscript for next line */
+		if (char_off != 0)
+		{
+			char_size[0]=nonscaled_size[0];
+			char_off=0;
+		}
 		break;
 	    }
 
@@ -849,8 +954,45 @@ process_direc_print(cp)
 		fprintf(fp, "%f setgray\n", 1.0 - (float)cp->cti_value /100.0);
 		break;
 
-	default:
+    /* setup for super/subscript */
+    case CTL_SETSUP:
+                if (cp->cti3_value1 > 100 || cp->cti3_value1 < 10){
+                     sup_off = DEFAULT_SUPOFF;
+                }else{
+                     sup_off = cp->cti3_value1 / 100.;
+                }
+                if (cp->cti3_value2 > 100 || cp->cti3_value2 < 10){
+                     sub_off = DEFAULT_SUBOFF;
+                }else{
+                     sub_off = cp->cti3_value2 / 100.;
+                }
+                if (cp->cti3_value3 > 100 || cp->cti3_value3 < 10){
+                     sup_scale = DEFAULT_SUPSCALE;
+                }else{
+                     sup_scale = cp->cti3_value3 / 100.;
+                }
+                break;
+
+    /* subscript */
+    case CTL_SUB:
+		/* save old size and set new one */
+		nonscaled_size[0] = char_size[0];
+		char_size[0] = char_size[0] * sup_scale;
+		char_off = nonscaled_size[0] * (-sub_off);
+		fprintf(fp, "%d setcharsize\n", char_size[0]);
 		break;
+
+    /* superscript */
+    case CTL_SUP:
+		/* save old size and set new one */
+		nonscaled_size[0] = char_size[0];
+		char_size[0] = char_size[0] * sup_scale;
+		char_off = nonscaled_size[0] * sup_off;
+		fprintf(fp, "%d setcharsize\n", char_size[0]);
+		break;
+
+	default:
+	  break;
 	}
 }
 
@@ -1023,7 +1165,17 @@ line_end()
 		if (textpool[i].text) {
 			if (!textpool[i].pfx)
 				fprintf(fp, "updatefillzero ");
+
+			/* for super/subscript: modify the current y-position */
+			if (textpool[i].yoffset != 0)
+				fprintf(fp, "currentpoint %d add moveto ",
+					textpool[i].yoffset);
 			fprintf(fp, "show\n");
+
+			/* for super/subscript: reset the old y-position */
+			if (textpool[i].yoffset != 0)
+				fprintf(fp, "currentpoint %d sub moveto ",
+					textpool[i].yoffset);
 		} else {
 			fprintf(fp, "\n");
 			if (textpool[i].font) {
@@ -1426,6 +1578,7 @@ text_remember(text, ctype, fontsize, offset, pfx)
 	}
 	textpool[ntextpool].pfx = pfx;
 	textpool[ntextpool].xoffset = offset;
+	textpool[ntextpool].yoffset = char_off;
 	textpool[ntextpool].xsiz = linewidth - offset;
 	textpool[ntextpool].size = fontsize;
 	textpool[ntextpool].font = curfont[ctype];
@@ -1664,7 +1817,7 @@ static void
 print_usage(name)
 	char	*name;
 {
-	fprintf(stderr, "Usage: %s [-cirVt] [-f psfile] [-p paper] "
+	fprintf(stderr, "Usage: %s [-mcirVt] [-f psfile] [-p paper] "
 		"[-x xmargin] [-y ymargin] [-e encoding] mgpfile\n", name);
 	exit(0);
 }
@@ -1784,9 +1937,9 @@ print_32_image(print_image, visual)
 		for (x = 0; x < width; x ++) {
 			unsigned long pix;
 			pix = XGetPixel(print_image, x, y);
-			r =  pix & visual->red_mask & 0xff;
-			g = (pix & visual->green_mask >> 8)  & 0xff;
-			b = (pix & visual->blue_mask  >> 16) & 0xff;
+			r = ((pix & visual->red_mask)   >> 16) & 0xff;
+			g = ((pix & visual->green_mask) >>  8) & 0xff;
+			b =   pix & visual->blue_mask          & 0xff;
 
 			if (reverse)
 				fprintf(fp, "%02x%02x%02x",
@@ -1973,13 +2126,20 @@ main(argc, argv)
 	char *progname;
 	static char pathbuf[MAXPATHLEN];
 
+#ifdef USE_SETLOCALE
+        setlocale(LC_CTYPE, "");
+#endif
+
 	progname = argv[0];
 
 	/* set default paper size */
 	paper = findpaper(DEFAULT_PAPER_SIZE);
 
-	while ((opt = getopt(argc, argv, "ciVrf:x:y:p:e:X:Pt")) != -1) {
+	while ((opt = getopt(argc, argv, "mciVrf:x:y:p:e:X:Pt")) != -1) {
 		switch (opt) {
+		case 'm':         /* Added by Arnd Schmitter 23.07.2004 */
+		  PauseMode=1;
+		  break;
 		case 'c':
 			colorps++;
 			break;
@@ -2190,7 +2350,21 @@ loadfont(font, name)
 			font->psfont, font->font);
 	}
 	if (font->lang == ASCII && encoding) {
-		fprintf(fp,
+		/*
+		 * Symbol font does not need latin1-encoding
+		 */
+		if (strcmp(font->psfont, "Symbol") == 0) {
+			fprintf(fp,
+				"/%s%s\n"
+				"	/%s findfont\n"
+				"		dup length dict begin\n"
+				"		{1 index /FID ne {def} {pop "
+				"pop} ifelse} forall\n"
+				"	currentdict end\n"
+				"definefont pop\n",
+				fontstring(font), "t", font->psfont);
+		} else {
+			fprintf(fp,
 "/%s%s\n"
 "	/%s findfont\n"
 "		dup length dict begin\n"
@@ -2199,6 +2373,8 @@ loadfont(font, name)
 "	currentdict end\n"
 "definefont pop\n",
 			fontstring(font), "t", font->psfont, encoding->defstr);
+		}
+
 		fprintf(fp, "/%s {/%s%s findfont exch scalefont setfont} def\n",
 			fontstring(font), fontstring(font), "t");
 	} else {
@@ -2280,8 +2456,7 @@ findttfont(ctrl, lang, font)
 	pfont->ctrl = ctrl;
 	pfont->lang = lang;
 	pfont->font = font;
-	pfont->psfont = (char *)malloc(string_len +1);
-	strncpy(pfont->psfont, string, string_len);
+	pfont->psfont = strdup(string);
 	pfont->fontid = 0;
 	pfont->loaded = 0;
 
@@ -2296,3 +2471,4 @@ findttfont(ctrl, lang, font)
 	return pfont;
 }
 #endif /* FREETYPE */
+
