@@ -26,7 +26,7 @@
  * SUCH DAMAGE.
  */
 /*
- * $Id: tfont.c,v 1.43 2002/01/12 01:56:25 nishida Exp $
+ * $Id: tfont.c,v 1.44 2003/09/12 02:04:55 nishida Exp $
  */
 
 #include "mgp.h"
@@ -627,6 +627,18 @@ CharToUnicode(code, registry)
 					TT_Get_CharMap(face[tfcuridx], i, &char_map);
 					return TT_Char_Index(char_map, (short) code);
 				}
+#endif
+				else {
+					/* symbol font */
+					if (platform == 3 && encoding == 0) {
+						code |= 0xf000;
+						TT_Get_CharMap(face[tfcuridx],
+								i, &char_map);
+						return TT_Char_Index(char_map,
+								code);
+					}
+				}
+#ifdef FREETYPE_CHARSET16
 			}
 		}
 #endif
